package edu.prlab.ecsimple.service;

import edu.prlab.ecsimple.common.exception.EntityNotFoundException;
import edu.prlab.ecsimple.domain.Item;
import edu.prlab.ecsimple.domain.TheOrder;
import edu.prlab.ecsimple.model.OrderModel;
import edu.prlab.ecsimple.model.ResponseModel;
import edu.prlab.ecsimple.repo.ItemRepo;
import edu.prlab.ecsimple.repo.OrderRepo;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
@Transactional
public class OrderServiceImpl implements OrderService {

    private final OrderRepo orderRepo;
    private final ItemRepo itemRepo;

    @Override
    public Optional<TheOrder> create(TheOrder theOrder) {
        return Optional.of(theOrder)
                .map(it -> orderRepo.save(theOrder));
    }

    @Override
    public Optional<Item> create(Item item) {
        return Optional.of(item)
                .map(it -> itemRepo.save(item));
    }

    @Override
    public void addItemToOrder(Integer orderId, String sku) {
        orderRepo.findById(orderId)
                .map(order -> itemRepo.findById(sku)
                        .map(item -> order.getItems().add(item))
                        .orElseThrow(() -> new EntityNotFoundException("No found this order")))
                .orElseThrow(() -> new EntityNotFoundException("No found this order"));
    }


    @Override
    public Optional<TheOrder> checkout(OrderModel orderModel) {
        return Optional.of(orderModel)
                .map(it -> orderRepo.save(TheOrder.builder()
                        .email(orderModel.getEmail())
                        .state("created")
                        .checkoutDate(LocalDateTime.now())
                        .build()))
                .map(order -> {
                    orderModel.getLineItems().stream()
                            .map(it -> create(it)
                                    .orElseThrow(() -> new RuntimeException("save item error")))
                            .forEach(it -> addItemToOrder(order.getIid(), it.getSku()));
                    return order;
                });
    }
}
